# -*- coding: UTF-8 -*-
import json
import sys
from datetime import datetime, date

"""
Маппер из сущности ukm в сущность artix
"""


class UkmToArtixTmcMapper(object):
    measure_mapper = dict()
    qty_options_mapper = dict()
    tmc_restrictions_mapper = dict()

    subexcise_to_paymentobject_by_egaisType = {
        "0": lambda subExcise: 2 if int(subExcise) == 1 else 0,
        "1": lambda subExcise: 2,
        "3": lambda subExcise: 31 if int(subExcise) == 1 else 33,
        "4": lambda subExcise: 31 if int(subExcise) == 1 else 33
    }

    opmode_by_egaisType = {
        "0": "0",
        "1": "128",
        "3": "0",
        "4": "0"
    }

    @classmethod
    def get_str_in_utf(cls, data):
        if data is not None:
            # во избежание проблем с кириллицей на python2
            if sys.version_info.major == 2:
                data = data.encode("UTF-8")
        return data

    @classmethod
    def get_measure_code(cls, measure_name):
        """
        получение кода единицы измерения из маппинга, указанного в настройках
        <код артикс>=<название из ukm>
        """
        measure_name = cls.get_str_in_utf(measure_name)
        if measure_name is not None:
            for k, v in cls.measure_mapper.items():
                if v == measure_name:
                    return k
        # по умолчанию возвращаем 1
        return 1

    @classmethod
    def get_paymentobject(cls, egaisType, subExcise):
        """
        egaisType == 0 && subExcise == 0 =>  0
        egaisType == 0 && subExcise == 1 =>  2
        egaisType == 1 && subExcise == 0 =>  2
        egaisType == 1 && subExcise == 1 =>  2
        egaisType == 3 && subExcise == 0 =>  33
        egaisType == 3 && subExcise == 1 =>  31
        egaisType == 4 && subExcise == 0 =>  33
        egaisType == 4 && subExcise == 1 =>  31
        """
        return cls.subexcise_to_paymentobject_by_egaisType.get(egaisType, lambda x: 0)(subExcise if subExcise else 0)

    @classmethod
    def get_qty_options(cls, measure_name):
        """
        <название из ukm>=<список опций для кол-ва товара>
        """
        measure_name = cls.get_str_in_utf(measure_name)
        if measure_name in cls.qty_options_mapper:
            opts = list(str(cls.qty_options_mapper.get(measure_name)).replace(" ", "").split(","))
            if len(opts) > 0:
                return {str(opt).lower(): 1 for opt in opts}
        return {}

    @classmethod
    def get_tmc_restrictions(cls, group_code):
        """
        <список кодов групп товаров>=<опции товара в формате json>
        """
        try:
            restrictions = UkmToArtixTmcMapper.tmc_restrictions_mapper
            groups = list()
            # собираем группы из строки
            for k in filter(lambda x: x is not None and len(x) > 0, restrictions.keys()):
                groups.extend(str(k).replace(" ", "").split(','))
            if group_code is not None and group_code in groups:
                # ищем строку, содержащую искомый код группы
                keys = list(filter(lambda x: group_code in x, restrictions.keys()))
                if len(keys) > 0:
                    # получаем json из строки и переводим в нижний регистр(в nes всё в lowercase)
                    res = json.loads(restrictions.get(keys[0]))
                    res = {str(k).lower(): v for k, v in res.items()}
                    return res
        except Exception:
            return {}
        return {}

    @classmethod
    def map_tmc(cls, ukm_tmcs, measure_map=None, quantity_options_mapper=None, tmc_restrictions_mapper=None):
        cls.measure_mapper = measure_map if measure_map is not None else {}
        cls.qty_options_mapper = quantity_options_mapper if quantity_options_mapper is not None else {}
        cls.tmc_restrictions_mapper = tmc_restrictions_mapper if tmc_restrictions_mapper is not None else {}
        for ukm in ukm_tmcs:
            name = ukm["name"].replace("&amp;", "&")
            barcodes = list()
            for bcode in ukm["barcode"]:
                barcode = {'name': name, 'code': ukm["article"],
                           "cquant": 1 if bcode["quantity"] and int(bcode["quantity"]) == 0 else bcode["quantity"],
                           "barcode": bcode["id"]}
                barcodes.append(barcode)
            tmc = dict()
            tmc['name'] = name
            tmc['articul'] = ukm["article"]
            tmc['inventcode'] = ukm["article"]
            tmc['inventgroup'] = ukm.get("groupId", None)
            tmc['barcodes'] = barcodes
            tmc['measurecode'] = UkmToArtixTmcMapper.get_measure_code(ukm["measure"])
            tmc['taxgroupcode'] = ukm.get("taxgroupId", None)
            tmc['paymentobject'] = UkmToArtixTmcMapper.get_paymentobject(ukm.get("egaisType", 0),
                                                                         ukm.get("SubExcise", 0))
            tmc["barcode"] = None
            tmc["price"] = 0.0
            qty_options = UkmToArtixTmcMapper.get_qty_options(ukm["measure"])
            restrictions = UkmToArtixTmcMapper.get_tmc_restrictions(ukm.get("groupId", None))
            inventitemoptions = {}
            tmctype = 0
            if len(restrictions) > 0:
                # не все поля относятся к inventitemoptions
                tmctype = restrictions.pop("tmctype", 0)
                tmc["age"] = restrictions.pop("age", None)
                if "alcoholpercent" in restrictions:
                    tmc["alcoholpercent"] = float(restrictions.pop("alcoholpercent"))
                inventitemoptions = restrictions
            tmc["tmctype"] = tmctype
            tmc["options"] = {"quantityoptions": qty_options, "inventitemoptions": inventitemoptions}
            yield tmc

    @classmethod
    def map_prices(cls, ukm_prices):
        """
        маппинг переоценок
        """
        for price_data in ukm_prices:
            price = dict()
            price['price'] = price_data["value"]
            price['minprice'] = price_data["minprice"]
            price['minretailprice'] = price_data["mrc"] if price_data["mrc"] else price_data["minprice"]
            price['barcode'] = price_data["article"]
            price['documentid'] = 1
            price['doctype'] = 1
            price['effectivedate'] = datetime.combine(date.today(), datetime.min.time()).strftime("%Y-%m-%d %H:%M:%S")
            yield price

    @classmethod
    def map_tprices(cls, ukm_prices):
        """
        маппинг доп.цен
        """
        return ({'price': price["value"],
                 'pricecode': price["code"],
                 'name': price["name"],
                 'barcode': price["article"],
                 'documentid': 1,
                 'effectivedate': datetime.combine(date.today(), datetime.min.time()).strftime("%Y-%m-%d %H:%M:%S")
                 } for price in ukm_prices)

    @classmethod
    def map_alcocode(cls, ukm_alcocodes):
        """
        маппинг алкокодов
        """
        return ({"alcocode": alcocode["AlcCode"], 'tmccode': alcocode["article"]} for alcocode in ukm_alcocodes)

    @classmethod
    def map_groups(cls, ukm_groups):
        """
        маппинг групп товаров
        """
        return ({'groupcode': group["id"],
                 "parentgroupcode": group["owner"],
                 "groupname": group["name"].replace("&amp;", "&")} for group in ukm_groups)

    @classmethod
    def map_tax(cls, ukm_taxes):
        """
        маппинг налогов
        """
        return ({'code': tax["id"],
                 'name': tax["name"],
                 'rate': str(tax["percent"]).replace('%', '')} for tax in ukm_taxes)

    @classmethod
    def map_kit(cls, ukm_kits):
        """
        маппинг комплектов товаров и сущностей внутри них
        """
        kit_items = []
        for kit in ukm_kits:
            kit_items.extend(kit["subItem"])
        kits = ({'barcode': k["article"], 'name': k["article"]} for k in ukm_kits)
        kit_items = ({'tmccode': item["article"], 'quantity': item["quantity"], 'tmckitbarcode': item["parentcode"]} for
                     item in kit_items)
        return kits, kit_items

    @classmethod
    def map_cashier(cls, ukm_cashier):
        """
        маппинг пользователей кассы (mol, roleuser)
        """
        return ({'code': user["id"], 'name': UkmToArtixTmcMapper.get_str_in_utf(user["name"]),
                 'login': UkmToArtixTmcMapper.get_str_in_utf(user["name"]), 'password': user["password"],
                 "roleusers": [{'usercode': user["id"], 'rolecode': user["roleId"]}]} for user in ukm_cashier)

    @classmethod
    def map_picklist(cls, ukm_picklists):
        """
        маппинг каталогов визуального подбора (picklist)
        """
        return ({'code': item["code"], 'name': item["name"], 'tmccode': item.get("tmccode", None),
                 'parent': item.get("parent", None), 'image': item.get("image", None)}
                for item in ukm_picklists)
