# -*- coding: UTF-8 -*-

import traceback
from common import aif_objects, config, logger, statuses
from common.rest_state import RestState as rest_state
from common.statuses import upload as st
from upload.inserter import _Inserter, InserterError
from artix.filequeue.filequeuemanager import FileQueueManager


class Inserter(_Inserter):
    def __init__(self, node):
        _Inserter.__init__(self, node)
        self.fileQueueManager = FileQueueManager(self.node.inserter.queue.fileQueueRoot, self.node.inserter.queue.queueChunkSize, self.log, config.isCash())
        self.queueName = self.node.inserter.queue.queue
        self.errorObjectsCount = 0

    def start(self):
        try:
            if not self.queueName:
                self.queueName = 'artix.loaddict.%s' % self.node.id

            self.fileQueueManager.removeUncompletedTransactions(self.queueName, self.source if self.source else 'exchangers-upload')
            self.transactionId = self.fileQueueManager.beginTransaction(self.queueName)
            statuses.setUploadStatus(st.TRANSACTION_START)
            self.errorObjectsCount = 0
            return True
        except Exception as e:
            statuses.setUploadStatus(st.ERROR_TRANSACTION_START)
            self.log.error(u'Ошибка: %s' % str(e))
            self.log.error(traceback.format_exc())
        return False
    
    def add_package(self, listObjects):
        self.errorObjectsCount += aif_objects.checkObjects(listObjects, self.log)
        self.objectsCount += len(listObjects)
        self.log.debug(u'Отправляем пакет объектов справочника в очередь')
        statuses.setUploadStatus(st.ADD_PACKAGE)
        self.fileQueueManager.enqueue(self.queueName, self.transactionId,
                                      {'properties': {'type': 'datagram'}, 'content': listObjects})

    def stop(self):
        if not self.objectsCount:
            self.log.warning(u'Ни одной записи не отправлено в очередь')
            return self.rollback()
        prop = {'type': 'control', 'control': 'end_tran', 'source': 'exchangers-upload'}
        if not config.isCash():
            prop['dictBackOfficeId'] = self.node.currentConverter.dict_state_id
        else:
            prop['dictCashLoadStateId'] = self.node.currentConverter.dict_state_id
            prop['cashCode'] = self.node.id
        prop['statusServer'] = self.node.currentConverter.statusServer if hasattr(self.node.currentConverter, 'statusServer') else 'localhost:8080'
        prop['transactionId'] = self.transactionId
        prop['objectsCount'] = self.objectsCount

        self.log.info(u'Отправляем сообщение - конец транзакции')
        self.fileQueueManager.endTransaction(self.queueName, self.transactionId, {'properties': prop})

        if not config.isCash():
            rest_state.startLoadNES(self.log, self.node.id, self.node.nesServer)

        statuses.setUploadStatus(st.TRANSACTION_END)
        self.log.info(u'Загрузка прошла успешно. Разблокируем загрузку в очередь для магазина - %s' % self.node.id)
        return True

    def invalidObjectsCount(self):
        return self.errorObjectsCount

    def rollback(self):
        self.log.info(u'Отменяем загрузку справочников в очередь для магазина - %s' % self.node.id)
        self.fileQueueManager.removeTransaction(self.queueName, self.transactionId)
        return True
